local mod = ...



-- Building Asset Processor ------------------------------------------------------------------
mod:registerAssetProcessor("models/Cathedral4.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Building Part Type ------------------------------------------------------------------
mod:registerEnumValue("BUILDING_PART_TYPE", "BUILDING_PART_TYPE_SUPPORT")



-- Materials -------------------------------------------------------------------------------------
mod:registerAssetId("models/Cathedral4.fbx/Materials/Material.Stone1", "MATERIAL_SAINTANNECATHEDRAL_STONE1")
mod:registerAssetId("models/Cathedral4.fbx/Materials/Material.Stone2", "MATERIAL_SAINTANNECATHEDRAL_STONE2")
mod:registerAssetId("models/Cathedral4.fbx/Materials/Material.AccentStone1", "MATERIAL_SAINTANNECATHEDRAL_ACCENTSTONE1")
mod:registerAssetId("models/Cathedral4.fbx/Materials/Material.AccentStone2", "MATERIAL_SAINTANNECATHEDRAL_ACCENTSTONE2")



-- Register Scaling -------------------------------------------------------------------------------------
local function registerScalingA(mesh, part, partType, buildingZone, costEnd, costTiling, capacity, miniature, splendor)

	mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh .. ".Root", "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_ROOT")
	mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh .. ".End", "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END")
	mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh .. ".End.Constr", "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END_CONSTR")
	mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh .. ".Tiling", "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING")
	mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh .. ".Tiling.Constr", "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING_CONSTR")

    -- Basement
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_BASEMENT",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING",
        },
        Cost = costTiling,
    })

    -- Tiling
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING"
        },
        ConstructionVisual = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING_CONSTR",
        Cost = costTiling,
    })

    -- End
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END"
        },
        ConstructionVisual = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END_CONSTR",
        Cost = costEnd,
    })


    -- Final
    mod:registerAsset({
        DataType = "BUILDING_PART",
		Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part,
		Name = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_NAME",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_SCALER",
            CoreObjectPrefab = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_ROOT",
            EndPart = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END",
            FillerList = {
                "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_TILING",
            },
            BasementFillerList = {
                "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_BASEMENT",
            },
            MinimumScale = 0,
            IsVertical = true,
            IsRepeatLastFiller = true,
            MiniatureConfig = miniature,        },
        BuildingZone = buildingZone,
        Category = partType,
        EstateSplendor = {
            { Estate = "CLERGY", Quantity = splendor },
        },
        DesirabilityModifiers = {
            {
                Desirability = "DESIRABILITY_RESIDENTIAL",
                Type = "ADD",
                Radius = 60,
                Value = 0.25,
            },
        },
    })

    -- Capacity
    if (capacity) then
        if (capacity > 0) then
            mod:overrideAsset({
                Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_END",
                Capacity = capacity,
                AssetBuildingFunction = "BUILDING_FUNCTION_PUBLIC_LOUNGE_ROOM",
            })
        end
    end

end

-- Facing down with x pointing up in Blender: y increases going left, x increases going down
local poly1 = { -- y, x
	{ 4.0, 0.0 },
    { -4.0, 0.0 },
    { -4.0, -7.5 },
    { -2.0, -9.8 },
    { 2.0, -9.8 },
    { 4.0, -7.5 },
}

local buildingZoneCore1 = { ZoneEntryList = { { Polygon = polygon.createRectangle( {10, 16}, {0, 0} ), Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } } } }
local buildingZoneCore2 = { ZoneEntryList = { { Polygon = polygon.createRectangle( {5, 16}, {2.5, 0} ), Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } } } }
local buildingZoneCore3 = { ZoneEntryList = { { Polygon = poly1, Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } } } }
local buildingZoneCore4 = { ZoneEntryList = { { Polygon = polygon.createRectangle( {10, 12}, {0, 0} ), Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } } } }

local buildingZoneTower1 = { ZoneEntryList = { { Polygon = polygon.createRectangle( {5.6, 5.6}, {0, 0} ), Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } } } }
local buildingZoneButtress1 = { ZoneEntryList = { { Polygon = polygon.createRectangle( {1, 7.5}, {4.75, 0} ),Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }, }, }

local miniatureDefault = {
    ClipUnderGround = true,
    GroupHeight = 0,
    CameraPosition = { 0, 0, 0 },
    OrientationOffset = 0,
    CameraPitchOffset = 0.0,
}

local miniatureTower = {
    ClipUnderGround = true,
    GroupHeight = 0,
    CameraPosition = { 0, 18, -30 },
    OrientationOffset = 0,
    CameraPitchOffset = 0.0,
}

local costCore1 = {
    BuildRightTaxes = {
        { Resource = "GOLD_COINS", Quantity = 100 }
    },
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 50 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 40 },
            { Resource = "STONE", Quantity = 40 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "TOOL", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
        },
    },
}

local costCore2 = {
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 20 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 10 },
            { Resource = "STONE", Quantity = 10 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "TOOL", Quantity = 10 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
        },
    },
}

local costCore3 = {
    BuildRightTaxes = {
        { Resource = "GOLD_COINS", Quantity = 20 }
    },
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 20 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 10 },
            { Resource = "STONE", Quantity = 10 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "TOOL", Quantity = 10 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
        },
    },
}

local costCore4 = {
    BuildRightTaxes = {
        { Resource = "GOLD_COINS", Quantity = 20 }
    },
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 10 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 40 },
            { Resource = "STONE", Quantity = 40 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "TOOL", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 10 },
        },
    },
}

local costTower = {
    BuildRightTaxes = {
        { Resource = "GOLD_COINS", Quantity = 100 }
    },
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 100 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 40 },
            { Resource = "STONE", Quantity = 40 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "TOOL", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "PLANK", Quantity = 20 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 10 },
            { Resource = "PLANK", Quantity = 10 },
            { Resource = "TOOL", Quantity = 10 },
        },
    },
}

local costButtress = {
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 5 }
    },
    ResourceNeededList = {
        {
            { Resource = "STONE", Quantity = 10 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 10 },
            { Resource = "TOOL", Quantity = 5 },
        },
    },
}

local costCoreTiling = {
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 10 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 10 },
            { Resource = "POLISHED_STONE", Quantity = 20 },
        },
        {
            { Resource = "TOOL", Quantity = 10 },
            { Resource = "PLANK", Quantity = 10 },
        },
    },
}

local costTowerTiling = {
    -- BuildRightTaxes = {
    --     { Resource = "GOLD_COINS", Quantity = 20 }
    -- },
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 20 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 10 },
            { Resource = "POLISHED_STONE", Quantity = 20 },
            { Resource = "TOOL", Quantity = 10 },
            { Resource = "PLANK", Quantity = 10 },
        },
    },
}

-- local function registerScalingA(mesh, part, type, buildingZone, costEnd, costTiling, capacity, miniature)
registerScalingA("Core.Part.001", "CORE1", "CORE", buildingZoneCore1, costCore1, costCoreTiling, 40, miniatureDefault, 2.0)
registerScalingA("Core.Part.002", "CORE2", "CORE", buildingZoneCore2, costCore2, costCoreTiling, 20, miniatureDefault, 1.0)
registerScalingA("Core.Part.003", "CORE3", "CORE", buildingZoneCore3, costCore3, costCoreTiling, 10, miniatureDefault, 1.0)
registerScalingA("Core.Part.004", "CORE4", "CORE", buildingZoneCore4, costCore4, costCoreTiling, 20, miniatureDefault, 2.0)
registerScalingA("Tower.Part.001", "TOWER1", "BELL_TOWER", buildingZoneTower1, costTower, costTowerTiling, nil, miniatureTower, 5.0)
registerScalingA("Tower.Part.002", "TOWER2", "BELL_TOWER", buildingZoneTower1, costTower, costTowerTiling, nil, miniatureTower, 5.0)
registerScalingA("Buttress.Part.001", "BUTTRESS1", "BUILDING_PART_TYPE_SUPPORT", buildingZoneButtress1, costButtress, costButtress, nil, miniatureDefault, 1.0)



-- Musical Part --------------------------------------------------------------------------------------
mod:overrideAsset({
    Id = "BUILDING_PART_SAINTANNECATHEDRAL_TOWER1",
    AssetBuildingFunction = "BUILDING_FUNCTION_LARGE_BELL",
})

mod:overrideAsset({
    Id = "BUILDING_PART_SAINTANNECATHEDRAL_TOWER2",
    AssetBuildingFunction = "BUILDING_FUNCTION_LARGE_BELL",
})



-- Simple assets -------------------------------------------------------------------------------------
local function registerSimpleA(mesh, part, partType, buildingZone, costPart, splendor, constr, glass)

	mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh, "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part)

    local prefabConstr = nil
    if constr then
        local prefabConstr = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_CONSTR"
        mod:registerAssetId("models/Cathedral4.fbx/Prefab/" .. mesh .. ".Constr", prefabConstr)
    end

    local partCategory = nil
    if partType ~= "MINOR" then
        partCategory = partType
    end

    mod:registerAsset({
		DataType = "BUILDING_PART",
		Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part,
		Name = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part .. "_NAME",
        BuildingZone = buildingZone,
		ConstructorData = {
			DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
			CoreObjectPrefab = "PREFAB_BUILDING_PART_SAINTANNECATHEDRAL_" .. part,
            MiniatureConfig = {
                ClipUnderGround = false,
                GroupHeight = 0,
                CameraPosition = { 0, 0, 0 },
                OrientationOffset = 0,
                CameraPitchOffset = 0.0,
            },
        },
        ConstructionVisual = prefabConstr,
        Cost = costPart,
        IsOnlyAttached = true,
        EstateSplendor = {
            { Estate = "CLERGY", Quantity = splendor },
        },
        Category = partCategory,
	})

    -- Building Part Type
    if (partType ~= "") then

        mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/" .. mesh, {
            DataType = "COMP_BUILDING_PART",
            BuildingPartType = partType,
        })

    end

    -- Building Part Type
    if glass then

        mod:overrideAsset({
            Id = "BUILDING_PART_SAINTANNECATHEDRAL_" .. part,
            CraftedResourceNeeded = "STAINED_GLASS",
        })

    end

end

local buildingZoneDoor1 = { ZoneEntryList = { { Polygon = polygon.createRectangle( {6, 3}, {0, 1} ), Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } } } }

local costDoor = {
    -- BuildRightTaxes = {
    --     { Resource = "GOLD_COINS", Quantity = 20 }
    -- },
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 10 }
    },
    ResourceNeededList = {
        {
            { Resource = "WOOD", Quantity = 4 },
            { Resource = "STONE", Quantity = 10 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 10 },
            { Resource = "TOOL", Quantity = 10 },
        },
        {
            { Resource = "POLISHED_STONE", Quantity = 10 },
        },
    },
}

local costWindow1 = {
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 20 }
    },
    ResourceNeededList = {
        {
            { Resource = "GLASS", Quantity = 10 },
            { Resource = "TOOL", Quantity = 5 },
        },
    },
}

local costWindow2 = {
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 10 }
    },
    ResourceNeededList = {
        {
            { Resource = "GLASS", Quantity = 10 },
            { Resource = "TOOL", Quantity = 5 },
        },
    },
}

local costWindow3 = {
    UpkeepCost = {
        { Resource = "GOLD_COINS", Quantity = 5 }
    },
    ResourceNeededList = {
        {
            { Resource = "GLASS", Quantity = 10 },
            { Resource = "TOOL", Quantity = 5 },
        },
    },
}

-- function registerSimpleA(mesh, part, partType, attachType, buildingZone, costPart, glass)
registerSimpleA("Door.Part.001", "DOOR1", "DOOR", buildingZoneDoor1, costDoor, 1.0, true, false)
registerSimpleA("Window.Part.001", "WINDOW1", "MINOR", nil, costWindow1, 4.0, false, true)
registerSimpleA("Window.Part.002", "WINDOW2", "MINOR", nil, costWindow1, 4.0, false, true)
registerSimpleA("Window.Part.003", "WINDOW3", "MINOR", nil, costWindow3, 1.0, false, true)
registerSimpleA("Window.Part.004", "WINDOW4", "MINOR", nil, costWindow2, 1.0, false, true)



-- Grounding
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Door.Part.001", {DataType = "COMP_GROUNDED"})

-- mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Buttress.Part.001.End.Constr/Step_0.Buttress.Part.001.End", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Core.Part.001.End.Constr/Step_0.Core.Part.001.End", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Core.Part.002.End.Constr/Step_0.Core.Part.002.End", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Core.Part.003.End.Constr/Step_0.Core.Part.003.End", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Core.Part.004.End.Constr/Step_0.Core.Part.004.End", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Tower.Part.001.End.Constr/Step_0.Tower.Part.001.End", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Cathedral4.fbx/Prefab/Tower.Part.002.End.Constr/Step_0.Tower.Part.002.End", {DataType = "COMP_GROUNDED"})

-- for i=1, 4, 1 do
-- 	mod:registerPrefabComponent("models/FrontierChurch1.fbx/Prefab/Core1Construction/Step_0." .. string.sub("000" .. i, -3), {DataType = "COMP_GROUNDED"})
-- end



-- Collider Platform (climbable mesh)  -------------------------------------------------------------------------------------
mod:configurePrefabFlagList("models/Cathedral4.fbx/Prefab/Door.Part.001", { "PLATFORM" })



-- Building: Cathedral -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_CATHEDRAL",
	Name = "BUILDING_SAINTANNECATHEDRAL_NAME",
	Description = "BUILDING_SAINTANNECATHEDRAL_DESC",
	BuildingType = "SERVICE",
    AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
    AssetMiniatureBuildingPart = "BUILDING_PART_SAINTANNECATHEDRAL_TOWER1",
    AssetBuildingFunction = "BUILDING_FUNCTION_CHURCH",
	BuildingPartSetList = {
		{
			Name = "BUILDING_SET_SAINTANNECATHEDRAL1_NAME",
			BuildingPartList = { 
				"BUILDING_PART_SAINTANNECATHEDRAL_CORE1",
				"BUILDING_PART_SAINTANNECATHEDRAL_CORE4",
				"BUILDING_PART_SAINTANNECATHEDRAL_CORE2",
				"BUILDING_PART_SAINTANNECATHEDRAL_DOOR1",
				"BUILDING_PART_SAINTANNECATHEDRAL_TOWER1",
				"BUILDING_PART_SAINTANNECATHEDRAL_TOWER2",
				"BUILDING_PART_SAINTANNECATHEDRAL_CORE3",
				"BUILDING_PART_SAINTANNECATHEDRAL_BUTTRESS1",
			}
		},
		{
			Name = "BUILDING_SET_SAINTANNECATHEDRAL2_NAME",
			BuildingPartList = {
				"BUILDING_PART_SAINTANNECATHEDRAL_WINDOW3",
				"BUILDING_PART_SAINTANNECATHEDRAL_WINDOW4",
				"BUILDING_PART_SAINTANNECATHEDRAL_WINDOW1",
				"BUILDING_PART_SAINTANNECATHEDRAL_WINDOW2",
			}
		},
    },
	RequiredPartList = {
		{ Category = "CORE", Min = 1 },
		{ Category = "DOOR", Min = 1 },
		{ Category = "BELL_TOWER", Min = 1 },
	},
	VillagerRequired = { Status = "CITIZEN", Quantity = 10 },
	IsEditable = true,
	MaterialSetList = {
		{
			SetName = "MATERIAL_SET_SAINTANNECATHEDRAL1_NAME",
			MaterialList = {
				"MATERIAL_SAINTANNECATHEDRAL_STONE1",
				"MATERIAL_SAINTANNECATHEDRAL_ACCENTSTONE1",
			}
		},
		{
			SetName = "MATERIAL_SET_SAINTANNECATHEDRAL2_NAME",
			MaterialList = {
				"MATERIAL_SAINTANNECATHEDRAL_STONE2",
				"MATERIAL_SAINTANNECATHEDRAL_ACCENTSTONE2",
			}
		},
	},
})
